@@ -72,7 +72,14 @@ class _V2RequestStrategy(_RequestStrategy):
         super(_V2RequestStrategy, self).__init__(adap, **kwargs)
         self._client = v2_client.Client(session=adap)
 
-        self.verify_token = self._client.tokens.validate_access_info
+    def verify_token(self, token):
+        auth_ref = self._client.tokens.validate_access_info(token)
+
+        if not auth_ref:
+            msg = _('Failed to fetch token data from identity server')
+            raise exc.InvalidToken(msg)
+
+        return {'access': auth_ref}
 
     def _fetch_signing_cert(self):
         return self._client.certificates.get_signing_certificate()
@@ -92,10 +99,17 @@ class _V3RequestStrategy(_RequestStrategy):
         super(_V3RequestStrategy, self).__init__(adap, **kwargs)
         self._client = v3_client.Client(session=adap)
 
-        self.verify_token = functools.partial(
-            self._client.tokens.validate,
+    def verify_token(self, token):
+        auth_ref = self._client.tokens.validate(
+            token,
             include_catalog=self._include_service_catalog)
 
+        if not auth_ref:
+            msg = _('Failed to fetch token data from identity server')
+            raise exc.InvalidToken(msg)
+
+        return {'token': auth_ref}
+
     def _fetch_signing_cert(self):
         return self._client.simple_cert.get_certificates()
 
@@ -198,19 +212,25 @@ class IdentityServer(object):
         try:
             auth_ref = self._request_strategy.verify_token(user_token)
         except exceptions.NotFound as e:
-            self._LOG.warn(_LW('Authorization failed for token'))
-            self._LOG.warn(_LW('Identity response: %s'), e.response.text)
+            self._LOG.warning(_LW('Authorization failed for token'))
+            self._LOG.warning(_LW('Identity response: %s'), e.response.text)
+            raise exc.InvalidToken(_('Token authorization failed'))
         except exceptions.Unauthorized as e:
             self._LOG.info(_LI('Identity server rejected authorization'))
-            self._LOG.warn(_LW('Identity response: %s'), e.response.text)
+            self._LOG.warning(_LW('Identity response: %s'), e.response.text)
             if retry:
                 self._LOG.info(_LI('Retrying validation'))
                 return self.verify_token(user_token, False)
+            msg = _('Identity server rejected authorization necessary to '
+                    'fetch token data')
+            raise exc.ServiceError(msg)
         except exceptions.HttpError as e:
             self._LOG.error(
                 _LE('Bad response code while validating token: %s'),
                 e.http_status)
-            self._LOG.warn(_LW('Identity response: %s'), e.response.text)
+            self._LOG.warning(_LW('Identity response: %s'), e.response.text)
+            msg = _('Failed to fetch token data from identity server')
+            raise exc.ServiceError(msg)
         else:
             return auth_ref
 
